@@ -4,24 +4,22 @@ from __future__ import annotations
 
 import abc
 import bisect
-import collections
 import enum
 import functools
-import operator
 import typing
 from typing import (
     Any,
     Callable,
     Generic,
-    Hashable,
     Iterable,
     Iterator,
-    MutableMapping,
     NamedTuple,
     Sequence,
     TypeVar,
 )
 
+import toolz
+
 from .aggregator import Aggregate, Aggregator
 from .functions.associative import BinaryAssociativeAggregate, UnaryAssociativeAggregate
 from .functions.navigation import (
@@ -30,12 +28,13 @@ from .functions.navigation import (
     UnaryNavigationAggregate,
 )
 from .functions.ranking import RankingAggregate
+from .protocols import Comparable
 from .row import AbstractRow
 from .typehints import Following, OrderBy, OrderingKey, PartitionBy, Preceding, T
 
 
 class StartStop(NamedTuple):
-    """A class to hold start and stop values for a range."""
+    """A class to hold start and stop values for a range of rows."""
 
     start: int
     stop: int
@@ -413,32 +412,14 @@ class AggregateSpecification(Generic[ConcreteAggregate]):
     def __init__(
         self,
         aggregate_type: type[ConcreteAggregate],
-        getters: tuple[Getter, ...],
+        *getters: Getter,
     ) -> None:
         self.aggregate_type: type[ConcreteAggregate] = aggregate_type
         self.getters = getters
 
 
-def compute_partition_key(
-    row: AbstractRow, partition_by: Iterable[PartitionBy]
-) -> tuple[Hashable, ...]:
-    """Compute a partition key from `row` and `partition_by`.
-
-    Parameters
-    ----------
-    row
-        An :class:`~stupidb.row.AbstractRow` instance.
-    partition_by
-        An iterable of :class:`~typing.Callable` taking a single
-        :class:`~stupidb.row.AbstractRow` argument and returning a
-        :class:`Hashable` object.
-
-    """
-    return tuple(partition_func(row) for partition_func in partition_by)
-
-
 def row_key_compare(
-    order_by: Sequence[OrderBy],
+    order_func: Callable[[AbstractRow], tuple[Comparable[T], ...]],
     null_ordering: Nulls,
     left_row: AbstractRow,
     right_row: AbstractRow,
@@ -450,8 +431,7 @@ def row_key_compare(
     ``NULL`` ordering is handled using `null_ordering`.
 
     """
-    keys = ((order_func(left_row), order_func(right_row)) for order_func in order_by)
-    for left_key, right_key in keys:
+    for left_key, right_key in zip(order_func(left_row), order_func(right_row)):
         if left_key is None and right_key is not None:
             return null_ordering.value
         if left_key is not None and right_key is None:
@@ -469,7 +449,8 @@ def row_key_compare(
 
 
 def make_key_func(
-    order_by: Sequence[OrderBy], nulls: Nulls
+    order_func: Callable[[AbstractRow], tuple[Comparable[T], ...]],
+    nulls: Nulls,
 ) -> Callable[[AbstractRow], OrderingKey[T]]:
     """Make a function usable with the key argument to sorting functions.
 
@@ -483,7 +464,7 @@ def make_key_func(
         :class:`~stupidb.row.AbstractRow`.
 
     """
-    return functools.cmp_to_key(functools.partial(row_key_compare, order_by, nulls))
+    return functools.cmp_to_key(functools.partial(row_key_compare, order_func, nulls))
 
 
 class WindowAggregateSpecification(Generic[ConcreteAggregate]):
@@ -531,15 +512,8 @@ class WindowAggregateSpecification(Generic[ConcreteAggregate]):
 
         """
         frame_clause = self.frame_clause
-        partition_by = frame_clause.partition_by
         order_by = frame_clause.order_by
 
-        # A mapping from each row's partition key to a list of rows in that
-        # partition.
-        partitions: MutableMapping[
-            tuple[Hashable, ...], list[AbstractRow]
-        ] = collections.defaultdict(list)
-
         # Generate names for temporary order by columns, users never see these.
         #
         # TODO: If we had static schema information these wouldn't be necessary
@@ -550,34 +524,37 @@ class WindowAggregateSpecification(Generic[ConcreteAggregate]):
         # Add computed order by columns that are used when evaluating window
         # functions in range mode
         # TODO: check that if in range mode we only have single order by
+        order_func = toolz.juxt(*order_by)
         rows_for_partition = (
-            row.merge(
-                dict(
-                    zip(
-                        order_by_columns,
-                        (order_func(row) for order_func in order_by),
-                    )
-                )
-            )
-            for row in rows
+            row.merge(dict(zip(order_by_columns, order_func(row)))) for row in rows
         )
 
-        # partition
-        for row in rows_for_partition:
-            partitions[compute_partition_key(row, partition_by)].append(row)
+        # divide the input rows into partitions
+        #
+        # we only need the values once we've grouped, so there's no need to
+        # store a possibly expensive key like a tuple, so we store just the
+        # integer result of hash in the dict
+        #
+        # we also only need the partition values once the rows have
+        # been partitioned
+        partitions = toolz.groupby(
+            toolz.compose(hash, toolz.juxt(*frame_clause.partition_by)),
+            rows_for_partition,
+        ).values()
 
-        # sort
-        key = make_key_func(order_by, frame_clause.nulls)
-        for partition_key in partitions.keys():
-            partitions[partition_key].sort(key=key)
-
-        # (row_id, value) pairs containing the aggregation results
-        results: list[tuple[int, T | None]] = []
+        # aggregation results, preallocated to avoid the need to sort
+        # before returning: we later assign elements to this list using
+        # the original row id
+        results: list[T | None] = [None] * sum(map(len, partitions))
 
         # Aggregate over each partition
         aggregate_type = self.aggregate_type
         getters = self.getters
-        for partition_key, possible_peers in partitions.items():
+        key_func = make_key_func(order_func, frame_clause.nulls)
+        for possible_peers in partitions:
+            # sort the partition according to the ordering key
+            possible_peers.sort(key=key_func)
+
             # Construct an aggregator for the function being computed
             #
             # For navigation functions like lead, lag, first, last and nth, we
@@ -602,10 +579,9 @@ class WindowAggregateSpecification(Generic[ConcreteAggregate]):
                 start, stop = frame_clause.compute_window_frame(
                     possible_peers, row, row_id_in_partition, order_by_columns
                 )
-                result = aggregator.query(start, stop)
-                results.append((row._id, result))
+                # Assign the result to the position of the original row id
+                # because we processed them in partition order, which might not
+                # be the same as the input order.
+                results[row._id] = aggregator.query(start, stop)
 
-        # Sort the results in order of the child relation, because we processed
-        # them in partition order, which might not be the same. Pull out the
-        # second element of each element in the result.
-        return map(operator.itemgetter(1), sorted(results, key=operator.itemgetter(0)))
+        return iter(results)
