@@ -19,10 +19,9 @@ import operator
 from typing import Any, Callable, Iterable, Mapping
 
 import tabulate
+import toolz
 from public import private, public
-from toolz import curry
 
-from .aggregation import Window  # noqa: F401
 from .aggregation import (
     AggregateSpecification,
     FrameClause,
@@ -89,13 +88,9 @@ def get(name: str) -> Callable[[AbstractRow], T | None]:
 
 
 @private  # type: ignore[misc]
-class shiftable(curry):
+class shiftable(toolz.curry):
     """Shiftable curry."""
 
-    def __init__(self, *args: Any, **kwargs: Any) -> None:
-        super().__init__(*args, **kwargs)
-        self.__annotations__ = self.func.__annotations__
-
     @property
     def __signature__(self) -> inspect.Signature:
         return inspect.signature(self.func)  # pragma: no cover
@@ -125,7 +120,12 @@ def table(rows: Iterable[Mapping[str, Any]]) -> Table:
     ... ]
     >>> t = table(rows)
     >>> t  # doctest: +ELLIPSIS
-    <stupidb.core.Relation object at 0x...>
+    name      balance
+    ------  ---------
+    Bob          -300
+    Bob          -100
+    Alice         400
+    Alice         700
 
     """
     return Table.from_iterable(rows)
@@ -145,7 +145,7 @@ def cross_join(right: Relation, left: Relation) -> Join:
 
     Examples
     --------
-    >>> from stupidb import cross_join, table
+    >>> from stupidb import cross_join, select, table
     >>> rows = [
     ...     dict(name="Bob", balance=-300),
     ...     dict(name="Bob", balance=-100),
@@ -155,8 +155,24 @@ def cross_join(right: Relation, left: Relation) -> Join:
     >>> t = table(rows)
     >>> s = table(rows)
     >>> crossed = cross_join(t, s)
-    >>> crossed  # doctest: +ELLIPSIS
-    <stupidb.core.CrossJoin object at 0x...>
+    >>> crossed >> select(
+    ...     left_name=lambda row: row.left["name"],
+    ...     left_balance=lambda row: row.left["balance"],
+    ...     right_name=lambda row: row.right["name"],
+    ...     right_balance=lambda row: row.right["balance"]
+    ... )
+    left_name      left_balance  right_name      right_balance
+    -----------  --------------  ------------  ---------------
+    Bob                    -300  Bob                      -300
+    Bob                    -300  Bob                      -100
+    Bob                    -300  Alice                     400
+    Bob                    -300  Alice                     700
+    Bob                    -100  Bob                      -300
+    Bob                    -100  Bob                      -100
+    Bob                    -100  Alice                     400
+    Bob                    -100  Alice                     700
+    Alice                   400  Bob                      -300
+    Alice                   400  Bob                      -100
 
     """
     return CrossJoin(left, right)
@@ -178,7 +194,7 @@ def inner_join(right: Relation, predicate: JoinPredicate, left: Relation) -> Joi
 
     Examples
     --------
-    >>> from stupidb import cross_join, table
+    >>> from stupidb import inner_join, table
     >>> rows = [
     ...     dict(name="Bob", balance=-300),
     ...     dict(name="Bob", balance=-100),
@@ -187,9 +203,18 @@ def inner_join(right: Relation, predicate: JoinPredicate, left: Relation) -> Joi
     ... ]
     >>> t = table(rows)
     >>> s = table(rows)
-    >>> crossed = cross_join(t, s)
-    >>> crossed  # doctest: +ELLIPSIS
-    <stupidb.core.CrossJoin object at 0x...>
+    >>> t >> inner_join(
+    ...     s,
+    ...     lambda left, right: left["balance"] < right["balance"]
+    ... ) >> select(name=lambda r: r.left["name"], bal=lambda r: r.left["balance"])
+    name      bal
+    ------  -----
+    Bob      -300
+    Bob      -300
+    Bob      -300
+    Bob      -100
+    Bob      -100
+    Alice     400
 
     """
     return InnerJoin(left, right, predicate)
@@ -510,12 +535,14 @@ def over(
     ...     balance=lambda r: r.balance,
     ...     date=lambda r: r.date,
     ... ) >> order_by(lambda r: r.name, lambda r: r.date)
-    >>> pprint(list(avg_balance_per_person), width=79)  # noqa: E501
-    [Row({'name': 'Alice', 'balance': 400, 'date': datetime.date(2019, 2, 9), 'avg_balance': 400.0}),
-     Row({'name': 'Alice', 'balance': 300, 'date': datetime.date(2019, 2, 10), 'avg_balance': 350.0}),
-     Row({'name': 'Alice', 'balance': 100, 'date': datetime.date(2019, 2, 11), 'avg_balance': 266.6666666666667}),
-     Row({'name': 'Bob', 'balance': -150, 'date': datetime.date(2019, 2, 5), 'avg_balance': -150.0}),
-     Row({'name': 'Bob', 'balance': 200, 'date': datetime.date(2019, 2, 6), 'avg_balance': 25.0})]
+    >>> avg_balance_per_person
+    name      balance  date          avg_balance
+    ------  ---------  ----------  -------------
+    Alice         400  2019-02-09        400
+    Alice         300  2019-02-10        350
+    Alice         100  2019-02-11        266.667
+    Bob          -150  2019-02-05       -150
+    Bob           200  2019-02-06         25
 
     """
     return WindowAggregateSpecification(child.aggregate_type, child.getters, window)
@@ -684,7 +711,7 @@ def difference_all(right: Relation, left: Relation) -> DifferenceAll:
 
 @public  # type: ignore[misc]
 @shiftable
-def limit(limit: int, relation: Relation, *, offset: int = 0) -> Limit:
+def limit(limit: int | None, relation: Relation, *, offset: int = 0) -> Limit:
     """Return the rows in `relation` starting from `offset` up to `limit`.
 
     Parameters
@@ -699,8 +726,8 @@ def limit(limit: int, relation: Relation, *, offset: int = 0) -> Limit:
     """
     if offset < 0:
         raise ValueError(f"invalid offset, must be non-negative: {offset}")
-    if limit < 0:
-        raise ValueError(f"invalid limit, must be non-negative: {limit}")
+    if limit is not None and limit < 0:
+        raise ValueError(f"invalid limit, must be non-negative or None: {limit}")
     return Limit(relation, offset=offset, limit=limit)
 
 
@@ -714,7 +741,7 @@ def count(x: Callable[[AbstractRow], T | None]) -> AggregateSpecification:
         A column getter.
 
     """
-    return AggregateSpecification(Count, (x,))
+    return AggregateSpecification(Count, x)
 
 
 @public  # type: ignore[misc]
@@ -727,7 +754,7 @@ def sum(x: Callable[[AbstractRow], R | None]) -> AggregateSpecification:
         A column getter.
 
     """
-    return AggregateSpecification(Sum, (x,))
+    return AggregateSpecification(Sum, x)
 
 
 @public  # type: ignore[misc]
@@ -740,7 +767,7 @@ def total(x: Callable[[AbstractRow], R | None]) -> AggregateSpecification:
         A column getter.
 
     """
-    return AggregateSpecification(Total, (x,))
+    return AggregateSpecification(Total, x)
 
 
 @public  # type: ignore[misc]
@@ -753,7 +780,7 @@ def first(x: Callable[[AbstractRow], T | None]) -> AggregateSpecification:
         A column getter.
 
     """
-    return AggregateSpecification(First, (x,))
+    return AggregateSpecification(First, x)
 
 
 @public  # type: ignore[misc]
@@ -766,7 +793,7 @@ def last(x: Callable[[AbstractRow], T | None]) -> AggregateSpecification:
         A column getter.
 
     """
-    return AggregateSpecification(Last, (x,))
+    return AggregateSpecification(Last, x)
 
 
 @public  # type: ignore[misc]
@@ -784,25 +811,25 @@ def nth(
         Callable to compute the row offset of the window to return.
 
     """
-    return AggregateSpecification(Nth, (x, i))
+    return AggregateSpecification(Nth, x, i)
 
 
 @public  # type: ignore[misc]
 def row_number() -> AggregateSpecification:
     """Compute the row number over a window."""
-    return AggregateSpecification(RowNumber, ())
+    return AggregateSpecification(RowNumber)
 
 
 @public  # type: ignore[misc]
 def rank() -> AggregateSpecification:
     """Rank the rows of a relation based on the ordering key given in `over`."""
-    return AggregateSpecification(Rank, ())
+    return AggregateSpecification(Rank)
 
 
 @public  # type: ignore[misc]
 def dense_rank() -> AggregateSpecification:
     """Rank the rows of a relation based on the ordering key given in `over`."""
-    return AggregateSpecification(DenseRank, ())
+    return AggregateSpecification(DenseRank)
 
 
 @public  # type: ignore[misc]
@@ -827,7 +854,7 @@ def lead(
         input and thus the default can be computed relative to the current row.
 
     """
-    return AggregateSpecification(Lead, (x, n, default))
+    return AggregateSpecification(Lead, x, n, default)
 
 
 @public  # type: ignore[misc]
@@ -852,7 +879,7 @@ def lag(
         input and thus the default can be computed relative to the current row.
 
     """
-    return AggregateSpecification(Lag, (x, n, default))
+    return AggregateSpecification(Lag, x, n, default)
 
 
 @public  # type: ignore[misc]
@@ -865,7 +892,7 @@ def mean(x: Callable[[AbstractRow], R | None]) -> AggregateSpecification:
         A column selector.
 
     """
-    return AggregateSpecification(Mean, (x,))
+    return AggregateSpecification(Mean, x)
 
 
 @public  # type: ignore[misc]
@@ -878,7 +905,7 @@ def min(x: Callable[[AbstractRow], Comparable | None]) -> AggregateSpecification
         A column selector.
 
     """
-    return AggregateSpecification(Min, (x,))
+    return AggregateSpecification(Min, x)
 
 
 @public  # type: ignore[misc]
@@ -891,7 +918,7 @@ def max(x: Callable[[AbstractRow], Comparable | None]) -> AggregateSpecification
         A column selector.
 
     """
-    return AggregateSpecification(Max, (x,))
+    return AggregateSpecification(Max, x)
 
 
 @public  # type: ignore[misc]
@@ -908,7 +935,7 @@ def cov_samp(
         A column selector.
 
     """
-    return AggregateSpecification(SampleCovariance, (x, y))
+    return AggregateSpecification(SampleCovariance, x, y)
 
 
 @public  # type: ignore[misc]
@@ -921,7 +948,7 @@ def var_samp(x: Callable[[AbstractRow], R | None]) -> AggregateSpecification:
         A column selector.
 
     """
-    return AggregateSpecification(SampleVariance, (x,))
+    return AggregateSpecification(SampleVariance, x)
 
 
 @public  # type: ignore[misc]
@@ -934,7 +961,7 @@ def stdev_samp(x: Callable[[AbstractRow], R | None]) -> AggregateSpecification:
         A column selector.
 
     """
-    return AggregateSpecification(SampleStandardDeviation, (x,))
+    return AggregateSpecification(SampleStandardDeviation, x)
 
 
 @public  # type: ignore[misc]
@@ -951,7 +978,7 @@ def cov_pop(
         A column selector.
 
     """
-    return AggregateSpecification(PopulationCovariance, (x, y))
+    return AggregateSpecification(PopulationCovariance, x, y)
 
 
 @public  # type: ignore[misc]
@@ -964,7 +991,7 @@ def var_pop(x: Callable[[AbstractRow], R]) -> AggregateSpecification:
         A column selector.
 
     """
-    return AggregateSpecification(PopulationVariance, (x,))
+    return AggregateSpecification(PopulationVariance, x)
 
 
 @public  # type: ignore[misc]
@@ -977,7 +1004,7 @@ def stdev_pop(x: Callable[[AbstractRow], R | None]) -> AggregateSpecification:
         A column selector.
 
     """
-    return AggregateSpecification(PopulationStandardDeviation, (x,))
+    return AggregateSpecification(PopulationStandardDeviation, x)
 
 
 @public  # type: ignore[misc]
@@ -1015,12 +1042,10 @@ def pretty(
     See Also
     --------
     stupidb.api.show
+
     """
     return tabulate.tabulate(
-        (rows >> limit(n)) if n is not None else rows,
-        tablefmt=tablefmt,
-        headers=headers,
-        **kwargs,
+        limit(n, rows), tablefmt=tablefmt, headers=headers, **kwargs
     )
 
 
@@ -1040,5 +1065,6 @@ def show(rows: Relation, **kwargs: Any) -> None:
     See Also
     --------
     stupidb.api.pretty
+
     """
     print(pretty(rows, **kwargs))
